'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Folder ACL Report.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 10/8/2007
'
' This script examines ACLs on a folder tree (folders and files)
' and reports the ACLs with filters that make interpreting
' what is going on a bit easier.
'
' Usage:
'	CSCRIPT //NOLOGO "SCRIPTNAME.VBS" "Path To Root Folder" E|i
'	where E or explicit means "show only explicit ACEs"
'		which filters out inherited permissions, allowing
'		you to focus on what has changed or is different
'		below the parent container
' 	and where I or inherited means "show all ACEs" 
'		including inherited ACEs
'
' The OUTPUT is tab-delimited, so it can be redirected
' into an Excel-compatible text file ( >ACLReport.csv)
'
' By default, the script will report ALL ACEs on the root folder
' because those are important for understanding what's going On
' below it.  You can change that behavior by setting
' bRootShowPerms to FALSE in the script, but it's a good default.
'
' The format of the ACE report is identical to ICACLS.EXE
' so you can use ICACLS.EXE /? from a command prompt to 
' learn what the various codes mean.
'
' The script REQUIRES icacls.exe, which is available on
' newer versions of Windows (and can be copied to older versions)
' The script runs icacls to actually enumerate the permissions,
' since that's easier (and a supported method) to report permissions.
' The script's main job is to take the ICACLs output and filter it
' 	- To hide inherited ACEs 
'		Using the I or inherited argument on the command line)
'	- To ignore standard ACEs
'		Configuring aTranslatePerms, below, to ignore specific permissions
'		or particular security principals, e.g. to ignore any permission
'		assigned to the SYSTEM account
'	- To format as tab-delimited, with the object (file or folder) on each line
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit

Dim FSO, WSHShell
Dim aTranslatePerms, sRootFolder
Dim sCommand, sResults, iExitCode
Dim aResults, sResult
Dim sFolder
Dim sReportLine
Dim iPos, sNextLine, i
Dim bExplicitOnly
Dim bRootShowPerms

' CONFIGURATION BLOCK
'
' Enter the permissions that you wish to "TRANSLATE" OR IGNORE in the report.
' 
' Each permission is a PAIR.  The first element is the permission as 
' reported by running icacls.exe against the file or folder.
' The second element is what you want this script to display.
' Use an empty string ("") to indicate you wish the report to ignore that
' permission.
' 
' You can include partial strings, such as group or user names
'
aTranslatePerms = Array(_
	"NT AUTHORITY\System", "" , _	
	"BUILTIN\Administrators","", _
	"CREATOR OWNER:(OI)(CI)(IO)(F)","", _
	"CONTOSO\ACL_", "" )
	
' "NT AUTHORITY\System" 				Ignore all permissions assigned to System
' "BUILTIN\Administrators" 				Ignore all permissions assigned to Administrators
' "CREATOR OWNER:(OI)(CI)(IO)(F)"		Ignore the standard Creator Owner Full Control permission
' "CONTOSO\ACL_"						Ignore all permissions assigned to our role-based management groups
'										which all begin with an ACL_ prefix

' The next line is a 'script preference'
' that is used in the Translate_Perm Function
' If TRUE, it instructs the script to INCLUDE INHERITED permissions on the root folder
' even if the command line says to look at explicit permissions only.
' The thought is that, to really understand what's happening on a folder tree, you need to look
' at ALL permissions on the root folder
bRootShowPerms = True

' GET ARGUMENTS

If WScript.Arguments.Count <> 2 Then
	WScript.Echo "USAGE: cscript //nologo SCRIPTNAME.vbs ""path of root folder to scan"" explicit|inherited"
	WScript.Quit(501)
End If

' Get root folder from first argument
sRootFolder = WScript.Arguments(0)

' Get Explicit or Inherited from second argument
Select Case Ucase(Left(WScript.Arguments(1),1))
	Case "E"
		bExplicitOnly = True
	Case "I"
		bExplicitOnly = False
	Case Else
		WScript.Echo "USAGE: cscript //nologo SCRIPTNAME.vbs ""path of root folder to scan"" explicit|inherited"
		WScript.Quit(501)
End Select

Set FSO = CreateObject("Scripting.FileSystemObject")
Set WSHShell = CreateObject("WScript.Shell")

sCommand = "icacls.exe """ & sRootFolder & """ /T"
sResults = ""

aResults = Execute_Capture(sCommand, 10, True)
iExitCode = aResults(0)
If iExitCode <> 0 Then
	WScript.Echo "ERROR"
	WScript.Echo aResults(1) & VbCrLf & aResults(2)
	WScript.Quit (501)
End If

aResults = Split(aResults(1), VbLf)

WScript.Echo "FOLDER TREE FILTERED ACL REPORT"
For i = 0 to UBound (aResults)
	' Look at each line that was outputted by ICACLS
	sResult = aResults(i)
	
	If sResult > "" Then
		If Left(sResult,1) = " " Then
			' If there's a space at the beginning of a line,
			' it is a permission related to a folder from the previous line
			sReportLine = Translate_Perm(sResult)
			If sReportLine > "" then
				WScript.Echo sFolder & vbTab & Trim(sReportLine)
			End If
		Else
			' There's no space at the beginning of the line, so this
			' is a new folder
			' 
			' We look at the NEXT line to see how far it is "indented"
			' and use that to figure out where the folder name is on this line.
			'
			' This will fail if a folder has only ONE ACE in it's ACL, but the
			' odds of that happening were not worth coding for in this sample
			'
			sNextLine = aResults(i+1)
			iPos = InStr(sNextLine, Trim(aResults(i+1))) - 2
			
			If iPos > 0 Then
				' This is a new folder
				sFolder = Left(sResult, iPos)
				
				' Now process the rest of the line
				sReportLine = Translate_Perm(Mid(sResult, iPos+2))
				If sReportLine > "" then
					WScript.Echo sFolder & vbTab & Trim(sReportLine)
				End If
			Else
				' The last line of iCacls is a "result" line
				WScript.Echo
				WScript.Echo sResult
			End If
			
		End If
	End If		
Next

Function Translate_Perm(ByVal sPerm)
	' This function looks at a permission line from iCacls,
	' compares it against the translation array,
	' and returns the "translated" permission (if it's in the array)
	' or the original permission line (if there was no match in the array)
	' It also performs other logic to determine what, exactly, to report
	
	Dim sTranslatePerm, sSourcePerm, i, bIgnore
	bIgnore = False
	
	' Start by assuming we'll return exactly what was given to the function
	Translate_Perm = sPerm
	
	' First, make sure there is a security principal listed.
	' If there is an "ACCOUNT UNKNOWN" (SID that does not translate to a name),
	' ICACLS returns permissions but no name and no colon
	If InStr(sPerm, ":") = 0 Then
		Translate_Perm = "ACCOUNT UNKNOWN:" & Trim(sPerm)
		Exit Function
	End If
	
	' If we're ignoring inherited...
	If bExplicitOnly And InStr(sPerm,"(I)") > 0 Then
		bIgnore = True
		'DEBUG: WScript.Echo "Inherited"
	Else	
		For i = 0 To UBound(aTranslatePerms) Step 2
			sSourcePerm = aTranslatePerms(i)
			sTranslatePerm = aTranslatePerms(i+1)
			If InStr(1, sPerm, sSourcePerm, VBTEXTCOMPARE) > 0 Then
				If sTranslatePerm = "" Then 
					bIgnore = True 
				Else 
					Translate_Perm = sTranslatePerm
				End If
				Exit For
			End If
		Next
	End If
	
	' If we're showing all permissions on the root,
	' and it is the root, then exit with either the original sPerm,
	' set at the top of the function, or a translation
	If (sFolder = sRootFolder) And bRootShowPerms Then 
		'DEBUG: WScript.Echo "Showing Root"
		Exit Function
	End If
	
	If bIgnore Then
		' We have a specific reason why we're reutnring a blank result
		' - either the translation table had a "" for the permission or
		' - we're ignoring inherited
		Translate_Perm = ""
	End If
		
End Function

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 070918
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	
	Dim oExec, iTimer, iExitCode, sOut, sErr
	
	Set oExec = WSHShell.Exec(sCommand)
	
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function
