'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Foldler_Synch.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 9/17/2007
'
' Synchronizes two folders using Robocopy.
' Has many applications as described in the 
' Windows Administration Resource Kit
' See the resource kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Const Robocopy_OK_Synched = 00			' no errors and no copying - source and destination are synched
Const Robocopy_OK_Copied = 01			' one or more files were copied
Const Robocopy_OK_ExtraFilesOrDirs = 02	' extra files or directories were detected - examine the output log
Const Robocopy_OK_Mismatched = 04		' mismatched files or directories detected - examine the output log
Const Robocopy_ERR_Skipped = 08			' some files or folders not copied due to error or retry limit
Const Robocopy_ERR_Serious = 16			' no copying due to usage or access denied Error
Const FileAttrib_Hidden = 2
Const FileAttrib_System = 4

' Set global objects and variables
Set WSHShell = CreateObject("WScript.Shell")
Set FSO = CreateObject("Scripting.FileSystemObject")
sUser = WSHShell.ExpandEnvironmentStrings("%username%")
sUserProfile = WSHShell.ExpandEnvironmentStrings("%userprofile%")

'==========================================================================
' CONFIGURATION BLOCK

' ROBOCOPY CONFIGURATION
' Include path to robocopy if not in the system path (e.g. system32)
sRobocopy = "robocopy.exe"
' Robocopy options (/MIR is hardwired later in the code)
' The script uses /COPY:DATSO which copies all attributes
' including the Data, Attributes, Timestamps, 
' Security (NTFS ACLs), and Ownership.
' It is not using the U option (or /COPYALL) because
' standard users don't have rights to copy auditing settings
sRobocopyOptions = "/COPY:DATSO /R:1 /W:30"
' How long (in minutes) to allow Robocopy to run 
' before terminating the process
iRobocopyTimeout = 10

' TARGET (SERVER) BACKUP/SYNCH FOLDER CONFIGURATION
' The root folder on the server for user sync folders
' You may need to add logic to determine this path if your
' users are not under a single share or DFS folder.
sUserSyncRoot = "\\contoso.com\users\" & sUser & "\backups"

' LOCAL FOLDERS ALLOWED TO BE SYNCHED
' The next three variables control which folders are allowed
' to be synched. When synched, all child folders are included
' because the Robocopy /MIR switch is hard-coded in the routine
' The local root folders under which sync folders are allowed.
' Folders in other namespaces will not be able to be synch'ed.
sUserLocalRoot = sUserProfile
' These subfolders of the root folder are allowed to be synched.
' The result will be that sUserSyncRoot has a structure of these
' subfolders just as the local user profile does.
aSyncFolders = Array("Local Files","Music","Videos","Pictures","Desktop")
' If the following flag is True, users can synch any subfolder
' of the folders listed in aSyncFolders.
' If False, then users can only sync the "top level" folders--
' those listed in aSyncFolders.
bSyncSubFolders = True
' So the RESULT is that folders matching this pattern can
' be synched:
' <sUserLocalRoot>\<any one of aSyncFolders>\[subfolder]
' where [subfolder] can be synched only if bSyncSubFolders is True
' These three variables and the Source Folder Validation work together.

' LOGGING CONFIGURATION
' Name for synch log. 
sSyncLogFileName = "_Contoso Synchronization Log.txt"
' Set hidden attribute on synch Log
' Requires the folder option "Show hidden files and folders" to "see" the log
bHideLog = True
' Set hidden & system attribute on synch Log
' Requires deselecting the folder option 
' "Hide protected operating system files" to see the log
bSuperhideLog = False
' The log will be prepended with the results of each synch log.
' Specify the maximum size of the log, in KB
' When the log exceeds this size, the "oldest" 25% of the log is purged.
iSyncLogMaxSize = 2048 ' 2MB
' The following describes how sections can be recognized within the log.
' It is used when log overflow results in the need to purge the log,
' so that purging grabs entire sections. 
' If you change what is written to the logs in the code, 
' you should change this accordingly.
' BUT it is cosemetic, only, so if you make a mistake, 
' it will just mean the logs are chopped off in less aesthetic places. 
sLogSectionMarker = VbCrLf & VbCrLf & VbCrLf & String(80, "=")

' SELF MANAGEMENT CONFIGURATION
' The script supports adding a flag file to the folder to indicate
' a successful or failed synchronization attempt. 
sSuccessFlagFile = "_Contoso Synchronization Success.txt"
sFailureFlagFile = "_Contoso Synchronization Failure.txt"
' Code is included that causes the script to exit if the
' most recent successful synchronization was within the
' numer of days specified by iBackupInterval.
' That way, back up happens only if that interval has been
' reached or exceeded.  
' This appies only in non-interactive (cscript) mode
' to support automation of the script as a scheduled task
' or startup/logon/logoff/shutdown script.
' If the user launches the script manually, it will prompt the user
' but will allow syncrhonization.
iBackupInterval = 5
' END CONFIGURATION BLOCK
'==========================================================================

' DETERMINE WHETHER SCRIPT IS RUNNING INTERACTIVELY
' This uses a check to see if the script was called with cscript
' and assumes, if it was, that the script should not run interactively
' and, if it was not, that the script should run interactively.
' You may want to have more sophisticated logic here.
' If interactive, messages are popups.
' If not, they are WScript.Echo'ed.
bInteractive = (WScript.FullName <> WScript.Path & "\cscript.exe")

' EVALUATE SOURCE & DESTINATION
If Not bInteractive Then
	sSource = WScript.Arguments.Named("source")
	sTarget = WScript.Arguments.Named("target")
Else
	' Not interactive - called from WScript
	If WScript.Arguments.Count = 0 Then
		' The script has been double-clicked 
		' or otherwise called with no arguments.
		' This supports a scenario in which user runs the script "Back Up This".
		' We assume, then, that we are to back up the script's parent folder.
		sSource = FSO.GetParentFolderName(Wscript.ScriptFullName)
	Else
		' The script has been called from WScript with an argument.
		' This supports adding the script as a shell command
		' for folders in Explorer, by adding a key to the Folder key
		' in HKEY_CLASSES_ROOT,
		' where the command should be
		' wscript.exe <path to script> %1
		sSource = WScript.Arguments(0)
	End If
	
	' SOURCE FOLDER VALIDATION
	' Make sure that the specified source folder is allowed to be synched
	bAllowed = False
	For Each sSyncFolder In aSyncFolders
		' For each subfolder specified in aSyncFolders
		' underneath the root specified by sUserLocalRoot
		' check to see if there's a match with the sSource
		sAllowedFolder = sUserLocalRoot & "\" & sSyncFolder
		If StrComp(Left(sSource, Len(sAllowedFolder)), sAllowedFolder, vbTextCompare) = 0 Then
			If ((bSyncSubFolders) Or  _
				(Len(sSource) = Len(sAllowedFolder))) Then
				bAllowed = True
				Exit For
			End If
		End If
	Next
	If Not bAllowed Then
		sMessage =  "Synchronizing is not allowed for this folder" & VbCrLf & _
					"SOURCE: " & sSource
		Call Notify(sMessage, "ERROR SYNCRHONIZING", 30, 64)
		WScript.Quit(501)
	End If			
	sTarget = sUserSyncRoot & Mid(sSource, Len(sUserLocalRoot)+1)
End If

If Not FSO.FolderExists(sSource) Then
	sMessage =  "Source folder does not exist" & VbCrLf & _
				"SOURCE: " & sSource
	Call Notify(sMessage, "ERROR SYNCRHONIZING", 30, 64)
	WScript.Quit(501)
End If
If Not FolderPath_Create(sTarget) Then
	sMessage =  "Target folder does not exist and cannot be created" & VbCrLf & _
				"TARGET: " & sTarget
	Call Notify(sMessage, "ERROR SYNCRHONIZING", 30, 64)
	WScript.Quit(501)
End If

' Self-management. Check flag file for most recent successful synch.
sSuccessFlag = sSource & "\" & sSuccessFlagFile
sFailureFlag = sSource & "\" & sFailureFlagFile
If FSO.FileExists(sSuccessFlag) Then
	Set oFile = FSO.GetFile(sSuccessFlag)
	dLastSuccess = oFile.DateCreated
	iInterval = Int(dLastSuccess - Now())
	WScript.Echo "Days since last backup: " & iInterval
	WScript.Quit
End If

' Unhide the log to avoid write errors
sSyncLogFile = sSource & "\" & sSyncLogFileName
If FSO.FileExists(sSyncLogFile) Then 
	Set oFile = FSO.GetFile(sSyncLogFile)
	oFile.Attributes = 0
	Set oFile = Nothing
End If

' Notify that operation is about to begin
sMessage =  "SOURCE: " & sSource & VbCrLf & _
			"TARGET: " & sTarget & VbCrLf & VbCrLf 
If bInteractive Then
			sMessage = sMessage & _
			"Synchronization will begin in 30 seconds, " & _
			"or when you click OK." & VbCrLf & VbCrLf 
End If			
			sMessage = sMessage &  _
			"Synchronization will proceed in the background, " & _
			"and may take some time (up to " & iRobocopyTimeout & " minutes)." & VbCrLf & VbCrLf & _
			"You will be notified when synchronization is complete." & VbCrLf & VbCrLf & _
			"You may continue to work during synchronization, " & _
			"but you should not open or work with files in the source folder."
Call Notify(sMessage, "Synchronization Operation", 30, 128)

' Run Robocopy, capturing the time started and time finished			
sCommand = """" & sRobocopy & """ """ & sSource & """ """ & sTarget & """ /MIR " & sRobocopyOptions
dStartDate = Now()
aResults = Execute_Capture(sCommand, iRobocopyTimeout * 60, True)
dEndDate = Now()

iResults = aResults(0)
bError = False
sMessage = ""

If iResults = 0 Then
	' Folder is synchronized
	bError = False
ElseIf iResults < Robocopy_ERR_Skipped Then
	' Results from Robocopy indicate changes were made.
	' Anything between 1 and 7 indicates changes were made.
	' So that we don't have to look through the log, we will
	' run robocopy again and check to see that synchronization is complete
	sCommand = """" & sRobocopy & """ """ & sSource & """ """ & sTarget & """ /MIR /R:1 /W:30"
	aResultsTest = Execute_Capture(sCommand, iRobocopyTimeout * 60, True)
	If aResultsTest(0) = 0 Then
		' We're in synch now.
		bError = False
	Else
		' We're still not in synch. Something is wrong.
		bError = True
	End If
Else
	' Anything >=8 means there was in fact an error synchronizing
	bError = True
End If

If Not bError Then
	' FULLY SYNCHED
	sMessage =  "Synchronized" & VbCrLf & _
				"SOURCE: " & sSource & VbCrLf & "TARGET: " & sTarget
	sLogMsg = 	String(80,"=") & VbCrLf & _
				"SUCCESS: FULLY SYNCHRONIZED" & VbCrLf & _
				"Source:  " & sSource & VbCrLf & _
				"Target:  " & sTarget & VbCrLf & _
				"Start:   " & dStartDate & VbCrLf & _
				"Finish:  " & dEndDate & VbCrLf & _
				"Code:    " & iResults & VbCrLf & _
				String(80,"=") & VbCrLf & VbCrLf
Else
	' NOT FULLY SYNCHED
	sMessage =  "Did not successfully synchronize:" & VbCrLf & _
				sSource & VbCrLf & "TO" & VbCrLf & sTarget
	sLogMsg = 	String(80,"=") & VbCrLf & _
				"ERROR:   SYNCHRONIZATION PROBLEM (" & iResults & ")" & VbCrLf & _
				"Source:  " & sSource & VbCrLf & _
				"Target:  " & sTarget & VbCrLf & _
				"Start:   " & dStartDate & VbCrLf & _
				"Finish:  " & dEndDate & VbCrLf & _
				"Code:    " & iResults & VbCrLf & _
				VbCrLf & _
				String (80,"-") & aResults(1) & VbCrLf & _
				String (80,"-") & aResults(2) & VbCrLf & _
				VbCrLf & _
				String(80,"=") & VbCrLf & VbCrLf
End If	

' Write the message to the log
ret = WriteFile_Prepend(sSyncLogFile, sLogMsg, False)

If ret Then
	' Log successfully updated.
	' Purge Log
	ret = TextFile_Purge(sSyncLogFile, iSyncLogMaxSize, 25, _
						 "bottom", sLogSectionMarker)
	If ret <= 2 Then
		' Purged log successfully or no purge needed
	Else
		' Problem purging Log
		sMessage = 	sMessage & VbCrLf & VbCrLf & _
					"Error " & ret & " purging the synchronization log." & _
					VbCrLf & sSyncLogFile
		bError = True
	End If	
	
	' Make sure log is superhidden
	Set oLog = FSO.GetFile(sSyncLogFile)
	If bHideLog Then _
		oLog.Attributes = oLog.Attributes Or FileAttrib_Hidden
	If bSuperhideLog Then
		oLog.Attributes = oLog.Attributes Or FileAttrib_Hidden
		oLog.Attributes = oLog.Attributes Or FileAttrib_System
	End If
	
	' Copy synchronization log to destination.
	' The xcopy command is using only the /h option
	' not the /o or /x options (which copy security settings)
	' so that it can run more easily under user credentials
	sCommand = "xcopy.exe """ & sSyncLogFile & """ """ & sTarget & """" & _
			   " /h /y"
	aResults = Execute_Capture(sCommand, 10, True)
	If aResults(0) = 0 Then
		' Copied log to destination successfully
	Else
		' Error copying up the synch log
		sMessage = 	sMessage & VbCrLf & _
					"Error copying synchronization log to server:" & VbCrLf & sCommand & VbCrLf & _
					aResults(2)
		bError = True
	End If
	
Else
	' Could not update local copy of log
	sMessage = 	sMessage & VbCrLf & VbCrLf & _
				"Error updating log information in " & VbCrLf & sSyncLogFile & VbCrLf & _
	bError = True
	' Note the way the code is structured right now, the log file
	' will remain UNhidden at this point.  It is only hidden by the
	' code above if the log was successfully updated.
	' This allows for easier troubleshooting.
End If

If Not bError Then
	' All went well
	sMessage =  "SUCCESSFUL synchronization" & VbCrLf & VbCrLf & _
				sMessage
	Call Notify (sMessage, "Synchronization Notification", 60, 128)
Else
	sMessage = 	"ERROR encountered during synchronization" & VbCrLf & VbCrLf & _
				sMessage
	Call Notify (sMessage, "Synchronization Notification", 60, 64)
	WSHShell.Run ("notepad.exe """ & sSyncLogFile & """")
End If
	

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 070918
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	
	Dim oExec, iTimer, iExitCode, sOut, sErr
	
	Set oExec = WSHShell.Exec(sCommand)
	
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function

Function WriteFile_Prepend(ByVal FilePath, ByVal Text, ByVal Overwrite)
	WriteFile_Prepend = False
	
	Const ForReading = 1, ForWriting = 2, ForAppending = 8, TristateUseDefault = -2
	Dim FSO, oFile, IOMode, sFileText, oFileText
	
	If Overwrite Then 
		IOMode = ForWriting
	Else
		IOMode = ForAppending
	End If
	
	Set FSO = CreateObject("Scripting.FileSystemObject")
	If FSO.FolderExists( FSO.GetParentFolderName(FilePath) ) Then
		If FSO.FileExists(FilePath) Then
			Set oFile = FSO.OpenTextFile(FilePath, ForReading)
			sFileText = oFile.ReadAll
			Text = Text & VbCrLf & sFileText
			Set oFile = Nothing
		End If
		' On Error Resume Next
		Set oFile = FSO.CreateTextFile(FilePath, True, False)
		oFile.Write Text
		oFile.Close
		If Err Then
			Err.Clear
			WriteFile_Prepend = False
		Else
			WriteFile_Prepend = True
		End If
	End If
	
End Function

Function TextFile_Purge (ByVal FilePath, ByVal MaxSize, ByVal PurgePercent, _
	ByVal TopBottom, ByVal Marker)
	' If the file specified by FilePath is bigger than MaxSize,
	' the file's contents are truncated by a percentage approximately
	' equal to PurgePercent, either at the beginning (top) or end (bottom)
	' of the file.
	' Additionally, a marker can be set to specify text to find within
	' the file as the start/end of the area to purge
	
	Const ForReading = 1, ForWriting = 2, ForAppending = 8, TristateUseDefault = -2
	Dim FSO, oFile, sFileText, oFileText

	Select Case UCase(TopBottom)
		Case "TOP"
			bTop = True
		Case "BOTTOM"
			bTop = False
		Case Else
			' Error bad parameter
			TextFile_Purge = 4
			Exit Function
	End Select
	
	If (PurgePercent<1 Or PurgePercent >99) Then
		' Bad parameter
		TextFile_Purge = 4
		Exit Function
	End If
	
	
	Set FSO = CreateObject("Scripting.FileSystemObject")
	If Not FSO.FileExists( FilePath ) Then
		' File does not exist
		TextFile_Purge = 2
		Exit Function
	End If

	On Error Resume Next
	Set oFile = FSO.GetFile(FilePath)
	If Err.Number <> 0 Then
		' Error getting file
		TextFile_Purge = 3
		Err.Clear
		Exit Function
	End If
	On Error GoTo 0
	iFileSize = oFile.Size /1024 ' convert to MB
	
	If iFileSize < MaxSize Then
		' Have not reached maximum size yet
		' No purge needed
		TextFile_Purge = 0
		Exit Function
	End If
	MsgBox "FileSize: " & iFileSize/1024 & VbCrLf & "Limit:    " & MaxSize
	On Error Resume Next
	Set oFile = FSO.OpenTextFile(FilePath, ForReading)
	If Err.Number <> 0 Then
		' Error reading file
		TextFile_Purge = 4
		Err.Clear
		Exit Function
	End If
	On Error GoTo 0
	
	sFileText = oFile.ReadAll
	oFile.Close
	
	iLenText = Len(sFileText)
	iPurgeChrs = Int(iLenText * (PurgePercent/100))
	If bTop Then
		iStart = InStr(iPurgeChrs, sFileText, Marker, vbtextcompare)
		If iStart = 0 Then iStart = iPurgeChrs
		iEnd = iLenText
	Else
		iStart = 1
		iEnd = InStr(iLenText - iPurgeChrs, sFileText, Marker, vbtextcompare)
		If iEnd = 0 Then iEnd = iLenText - iPurgeChrs
		iEnd = iEnd - 1
	End If
	
	sFileText = Mid(sFileText, iStart, iEnd)
	On Error Resume Next
	Set oFile = FSO.CreateTextFile(FilePath, True, False)
	oFile.Write sFileText
	oFile.Close
	If Err.Number <> 0 Then
		' Error writing the purged log
		TextFile_Purge = 5
		' DEBUG: WScript.Echo Err.Description
		' DEBUG: WScript.Echo FilePath
		Err.Clear
	Else
		' Purged successfully
		TextFile_Purge = 1
		WriteFile = True
	End If
	On Error GoTo 0
	
End Function

Sub Notify(ByVal sMessage, ByVal sTitle, ByVal sTimeOut, ByVal iButtonType)
	If ((Not bInteractive) Or (sTimeOut = 0)) Then
		WScript.Echo String(80,"-") & VbCrLf & _
					 sTitle & VbCrLf & String(80,".") & VbCrLf &_
					 sMessage & VbCrLf & String(80,"-") & VbCrLf 
	Else
		Call PopUp(sMessage, sTitle, sTimeOut, iButtonType)
	End If

End Sub

Sub PopUp(ByVal sMessage, ByVal sTitle, ByVal sTimeOut, ByVal iButtonType)
	' Version 070709 *VARIATION* with button type
	' Displays a popup message for a specified period of time
	' INPUTS:	sMessage: message to display
	'			sTitle: title bar of popup window
	'			sTimeOut: how long to display the popup
	
	Dim wshShell, btnCode
	Set wshShell = CreateObject("WScript.Shell")
	btnCode = wshShell.Popup (sMessage, sTimeOut, sTitle, iButtonType)
End Sub

Function FolderPath_Create(FolderPath)
	' VERSION 070918
	' Creates a folder path specified by FolderPath
	' and returns True (success) or False (failure)
	'
	' Recursively create a folder path
	' This is an "inside out" recursion that starts with the lowest level,
	' works its way "up" to the nearest ancestor that exists, then
	' creates folders down from there as it works its way out of recursions
	'
	' INPUT:	FolderPath:			desired folder path
	' OUTPUT:	FolderPath_Create:	TRUE: 	success
	'								FALSE:	failure
	
	FolderPath_Create = False
	On Error Resume Next
	bFolderExists = FSO.FolderExists(FolderPath)
	If Err.Number <> 0 Then
		' This handles scenarios where a sharename in a UNC doesn't exist.
		' I was getting Out of Memory errors on the FolderExists method.
		FolderPath_Create = False
		Exit Function
	End If
	If Not bFolderExists Then
		' If the folder doesn't exist already, test its parent
		' (RECURSIVE CALL TO SELF)
		If FolderPath_Create(FSO.GetParentFolderName(FolderPath)) Then
			' Create the folder
			FolderPath_Create = True
			Call FSO.CreateFolder(FolderPath)
		End If
	Else
		' This level of folder does exist
		FolderPath_Create = True
	End If
End Function